/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import java.util.Collection;
import java.util.Iterator;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.ComponentContext;
import org.oasisopen.sca.ServiceReference;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation with
 * 1 service with interface Service1
 * 4 references 
 * - requiredRef (1..1) with interface Service1
 * - singledRef (0..1) with interface Service1 (intended to be unwired)
 * - multiRef (0..n) with interface Service1
 * - multiRef2 (0..n) with interface Service1  (intended to be unwired)
 * 
 * This implementation performs invocations of the getServiceReferences() method of the ComponentContext API and
 * checks that the results conform to a series of specification requirements
 */
@Service(Service1.class)
public class Service1CCgetServiceReferencesImpl implements Service1 {
	
	private ComponentContext componentContext;
	private String responseAccumulator = "";
	
	@Property
	public String serviceName = "service1";
	@Reference
	public Service1 requiredRef;		// 1..1 multiplicity
	@Reference(required=false)
	public Service1 singleRef;			// 0..1 multiplicity - *SHOULD NOT BE WIRED*
	@Reference(required=false)
	public Service1[] multiRef;			// 0..n multiplicity
	@Reference(required=false)
	public Service1[] multiRef2;		// 0..n multiplicity - *SHOULD NOT BE WIRED*

	public Service1CCgetServiceReferencesImpl() {
		super();
	}
	
	/**
	 * Setter for the ComponentContext used by this implementation
	 * @param theContext - the ComponentContext
	 */
	@Context
	public void setContext( ComponentContext theContext ) {
		componentContext = theContext;
	} // end method setContext

	public String operation1(String input) {
		responseAccumulator += serviceName + " operation1 invoked getServiceReferences";
		
		if( componentContext == null ) return responseAccumulator + " injected componentContext null";
		
		// JCA-TA-9022 - IllegalArgumentException thrown if called for an x..1 reference
		try {
		    componentContext.getServiceReferences(Service1.class, "requiredRef");
		    responseAccumulator += " did not get IllegalArgumentException for 1..1 ref";
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " expected IllegalArgumentException received for 1..1 ref";	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for 1..1 ref: " + e.toString();
		} // end try
		
		// JCA-TA-9024 - IllegalArgumentException thrown if named reference does not have the businessInterface specified
		try {
		    componentContext.getServiceReferences(Service2.class, "multiRef");
		    responseAccumulator += " did not get IllegalArgumentException for invalid businessInterface";
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " expected IllegalArgumentException received for invalid businessInterface";	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for invalid businessInterface: " + e.toString();
		} // end try	
		
		// JCA-TA-9023 - IllegalArgumentException thrown if component does not have the named reference
		try {
		    componentContext.getServiceReferences(Service1.class, "randomRef");
		    responseAccumulator += " did not get IllegalArgumentException invalid reference name";
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " expected IllegalArgumentException received for invalid reference name";
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for invalid reference name: " + e.toString();
		} // end try
		
		// JCA-TA-9021 - returns an empty Collection for a 0..n reference that is not wired
		try {
		    Collection<ServiceReference<Service1>> serviceRefs = componentContext.getServiceReferences(Service1.class, "multiRef2");
		    if( serviceRefs == null ) {
			    responseAccumulator += " unexpected null Collection for unwired 0..n reference";
		    } else {
		    	if ( serviceRefs.isEmpty() ) {
		    		responseAccumulator += " expected empty Collection for unwired 0..n reference";
		    	} else {
		    		responseAccumulator += " unexpected non-empty Collection for unwired 0..n reference";
		    	}
		    }
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " unexpected IllegalArgumentException received for unwired 0..n reference";	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for unwired 0..n reference: " + e.toString();
		} // end try
		
		// JCA-TA-9020 - returns a Collection of ServiceReferences, with one for each target service configured for the
		//               0..n reference
		try {
		    Collection<ServiceReference<Service1>> serviceRefs = componentContext.getServiceReferences(Service1.class, "multiRef");
		    if( serviceRefs == null ) {
			    responseAccumulator += " unexpected null Collection for wired 0..n reference";
		    } else {
		    	if ( serviceRefs.isEmpty() ) {
		    		responseAccumulator += " unexpected empty Collection for wired 0..n reference";
		    	} else {
		    		responseAccumulator += " expected non-empty Collection for wired 0..n reference";
		    		Iterator<ServiceReference<Service1>> iter = serviceRefs.iterator();
		    		while( iter.hasNext() ) {
		    			// Actually invoke the target service to verify that the returned ServiceReference is indeed a proxy for that service
		    			ServiceReference<Service1> targetRef = iter.next();
		    			Service1 target = targetRef.getService();
		    			responseAccumulator += " " + target.operation1( "dynamic" );
		    		} // end while
		    	} // end if
		    } // end if
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " unexpected IllegalArgumentException received for wired 0..n reference";
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for wired 0..n reference: " + e.toString();
		} // end try		

		return responseAccumulator;
	} // end operation1

} // end class
